@@ -426,7 +426,7 @@ class TgWrap():
 
     def _run_sync(
         self, source_path, target_path, auto_approve, include_lock_file, dry_run, clean_up,
-        source_stage=None, target_stage=None, source_domain=None,
+        excludes=[], source_stage=None, target_stage=None, source_domain=None,
         ):
         """ Run a sync copying files from a source to a target path """
 
@@ -457,9 +457,10 @@ class TgWrap():
             env_file_stmt   = "--exclude='env.hcl'" if source_stage != target_stage else "--include='env.hcl'"
             lock_file_stmt  = "--include='.terraform.lock.hcl'" if include_lock_file \
                 else "--exclude='.terraform.lock.hcl'"
+            excludes_stmt = ' '.join([f"--exclude={x}" for x in excludes])
 
             cmd = f"rsync -aim {dry_run_stmt} {clean_stmt} " + \
-                f"--include='terragrunt.hcl' {lock_file_stmt} {env_file_stmt} " + \
+                f"--include='terragrunt.hcl' {lock_file_stmt} {env_file_stmt} {excludes_stmt} " + \
                 "--exclude='.terragrunt-cache/' --exclude='.terraform/' " + \
                 "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
                 "--exclude='.DS_Store' " + \
@@ -870,21 +871,22 @@ class TgWrap():
                 if target_stage not in value['applies_to_stages']:
                     self.printer.verbose(f'Target stage {target_stage} not applicable for action {key}.')
                 else:
-                    self.printer.verbose(f'Found deployment step: {key}')
+                    source_stage = value['source_stage']
+                    self.printer.verbose(f'Found deployment step {key} using source stage {source_stage}')
 
-                    source_path = os.path.join(source_dir, value['source_stage'])
+                    source_path = os.path.join(source_dir, source_stage)
                     source_modules = {
                         entry:{} for entry in os.listdir(source_path) if os.path.isdir(os.path.join(source_path, entry))
                     }
                     self.printer.verbose(f'Found modules: {source_modules}')
 
-                    include_modules = value['include_modules'] if len(value['include_modules']) > 0 else source_modules
+                    include_modules = value['include_modules'] if len(value.get('include_modules', {})) > 0 else source_modules
                     self.printer.verbose(f'Include modules: {include_modules}')
 
                     for module, module_details in include_modules.items():
                         source_module = module_details.get('source', module)
                         target_module = module_details.get('target', module)
-                        if source_module in value['exclude_modules'] or source_module in substacks:
+                        if source_module in value.get('exclude_modules', []) or source_module in substacks:
                             self.printer.verbose(f'Exclude module {source_module}')
                         else:
                             deploy_actions[module] = {
@@ -896,16 +898,18 @@ class TgWrap():
                         self.printer.verbose(f'Found substack : {ss}')
 
                         source_path = os.path.join(
-                            source_dir, value['source_stage'], substack['source'], ''
+                            source_dir, source_stage, substack['source'], ''
                             )
                         target_path = os.path.join(
                             target_dir, substack['target'], ''
                             )
 
+                        exclude_modules =substack.get('exclude_modules', [])
                         if os.path.exists(source_path):
                             deploy_actions[f'substack -> {substack["target"]}'] = {
                                 "source": source_path,
                                 "target": target_path,
+                                "excludes": exclude_modules,
                             }
                         else:
                             self.printer.warning(f'Source path of substack does not exist: {source_path}')
@@ -926,6 +930,7 @@ class TgWrap():
                 self._run_sync(
                     source_path=value['source'],
                     target_path=value['target'],
+                    excludes=value.get('excludes', []),
                     include_lock_file=True,
                     auto_approve=True,
                     dry_run=dry_run,
